"""Automatically generated by hassfest.

To update, run python3 -m script.hassfest
"""


FLOWS = [
    "adguard",
    "ambiclimate",
    "ambient_station",
    "axis",
    "cast",
    "daikin",
    "deconz",
    "dialogflow",
    "emulated_roku",
    "esphome",
    "geofency",
    "gpslogger",
    "hangouts",
    "heos",
    "homekit_controller",
    "homematicip_cloud",
    "hue",
    "ifttt",
    "ios",
    "ipma",
    "iqvia",
    "life360",
    "lifx",
    "locative",
    "logi_circle",
    "luftdaten",
    "mailgun",
    "met",
    "mobile_app",
    "mqtt",
    "nest",
    "notion",
    "openuv",
    "owntracks",
    "plaato",
    "point",
    "ps4",
    "rainmachine",
    "simplisafe",
    "smartthings",
    "smhi",
    "somfy",
    "sonos",
    "tellduslive",
    "toon",
    "tplink",
    "tradfri",
    "twilio",
    "unifi",
    "upnp",
    "wemo",
    "wwlln",
    "zha",
    "zone",
    "zwave"
]
